<?php
// $Id$

/**
 * @file
 * Node callbacks for the camtasia_relay module.
 */

function camtasia_relay_check_sum_string($input) {
  $sum = 458289; // Starting Checksum Seed
  $multiplier = 3;
  $input_len = strlen($input);
  for ( $i = 0; $i < $input_len; $i++ ) {
     $sum += ord((string)$input[$i]) * $multiplier;
     $multiplier++;
  }
  return $sum;
}

function local_camtasia_relay_process_file($hash_checksum, $xml_file_path, $terms_array = NULL) {
  $log_msg = "RPC call made: camtasia_relay_process_file with parameters hash_checksum: $hash_checksum, xml_file_path: $xml_file_path, "; 
  foreach ($terms_array as $vocabulary_name => $term_name) { 
    $log_msg .= "Term: $vocabulary_name => $term_name ; "; 
  }
  watchdog('camtasia_relay', $log_msg , NULL, WATCHDOG_ERROR, NULL);
  $flag_node_update = FALSE;
  $dirloc = variable_get('camtasia_relay_dirloc', '');
  $file_path = $dirloc ."/". $xml_file_path;
  if (!is_file($file_path)) {
    $log_msg = "RPC call return: camtasia_relay_process_file could not locate file $file_path";
    watchdog('camtasia_relay', $log_msg , NULL, WATCHDOG_ERROR, NULL);
    return "Could not locate $file_path";
  }
  $xml = file_get_contents($file_path);
  //this creates the simple xml object
  $xmldata = new simplexmlelement($xml);
  $duration = (string)$xmldata->totalDuration;
  $record_time = (string)$xmldata->date;
  $title = (string)$xmldata->title;
  $stored_secret_key = (string)variable_get('camtasia_relay_shared_secret_key', '');
  
  $validation_checksum = camtasia_relay_check_sum_string( $stored_secret_key ) . camtasia_relay_check_sum_string( $duration ) . camtasia_relay_check_sum_string( $record_time ) . camtasia_relay_check_sum_string( $title );
  if ($validation_checksum != $hash_checksum) {
    $log_msg = "RPC call return: camtasia_relay_process_file validation_checksum: $validation_checksum";
    watchdog('camtasia_relay', $log_msg , NULL, WATCHDOG_ERROR, NULL);
    return "Access Denied";
  }

  $result = db_query('SELECT n.nid FROM {node} n WHERE n.type = '."'camtasia_relay'");
  
  while ($node = db_fetch_object($result)) {
    $node_obj = node_load($node->nid);
    global $user;
    $user->uid = 1;
    if (($node_obj->title == $xmldata->title)&&($node_obj->profile == $xmldata->profile)&&($node_obj->email == $xmldata->presenter->email)&&($node_obj->date == $xmldata->date)) {
      $filesize = filesize($file_path);
      $des_file = camtasia_relay_file_move($file_path, $xmldata->presenter->userName, $node_obj->profile);
      camtasia_relay_file_attach($des_file, $filesize, $node_obj);
      
      $xml_files = $xmldata->outputFiles->fileList->file;
      foreach ($xml_files as $xml_file) {
        $output_file_name = $xml_file['name'];
        $output_file_path = dirname($file_path) ."/". $output_file_name;
        if (file_exists($output_file_path)) {
          $filesize = filesize($output_file_path);
          $des_file = camtasia_relay_file_move($output_file_path, $xmldata->presenter->userName, $node_obj->profile);
          camtasia_relay_file_attach($des_file, $filesize, $node_obj);
        }
      }
      $flag_node_update = TRUE;
      node_save($node_obj);
      return url("node/$node_obj->nid", array('absolute' => TRUE));
    }
  }
  
  if ($flag_node_update == FALSE) {
    $newnode = camtasia_relay_node_create($xmldata, $file_path, 1, 1, 0, 'camtasia_relay');
    if ($newnode==0) {
      return "Error in creating node";
    }
    //Relay3
    $node_terms = array();
    foreach ($terms_array as $vocabulary_name => $term_name) { 
      $vid = NULL;
      $vocab_object = camtasia_relay_get_vocabulary_by_name($vocabulary_name);
      if ($vocab_object == NULL) {
        //Vocabulary doesn't exist. Create Vocabulary
        $vid = create_vocabulary($vocabulary_name);
      }
      else{
        $vid = $vocab_object->vid;
      }
      $node_terms[$term_name]=$vid;
    }
    attach_term2node($node_terms, $newnode);
    node_save($newnode);
    return url("node/$newnode->nid", array('absolute' => TRUE));
  }
  return "Error in Content Creation";
}

function camtasia_relay_get_vocabulary_by_name($vocabulary_name) {
  $vocabs = taxonomy_get_vocabularies(NULL);
  foreach ($vocabs as $vocab_object) {
    if ($vocab_object->name == $vocabulary_name) {
      return $vocab_object;
    }
  }
  return NULL;
}

function create_vocabulary($vocabulary_name) {
  $vocabulary = array(
      'name' => $vocabulary_name, 
      'multiple' => 0, 
      'required' => 0, 
      'hierarchy' => 1, 
      'relations' => 0, 
      'module' => 'camtasia_relay', 
      'weight' => -10, 
      'nodes' => array('camtasia_relay' => 1),
    );
  taxonomy_save_vocabulary($vocabulary);
  return  $vocabulary['vid'];
} 
  
 /**
   * check_terms function
   * returns TRUE if term is found in data base, else FALSE.
   * @param string $term_search
   * @param string $vid
   * @return TRUE/FALSE
   */
function check_terms($term_search, $vid) {
  $result = db_query("select t.* from {term_data} t where t.vid = %d", $vid);
  while ($row = db_fetch_array($result)) {
    if ($row['name'] == $term_search) {
      return TRUE;
    }
  }
  return FALSE;
}
  
  /**
   * create_term function
   * Creates new term with specifid name and in specified vocabulary.
   * @param string $termname
   * @param string $vid
   */
function create_term($termname, $vid) {
  $new_term = array(
    'name' => $termname, 
    'parent' => 0, 
    'vid' => $vid, 
  );
  taxonomy_save_term($new_term);
}

  /**
   * attach_term2node function
   * Attaches terms to node.
   * @param string $node_terms
   * @param string $newnode
   */
function attach_term2node($node_terms, $newnode) {
  foreach ($node_terms as $key => $value) {
    $match = check_terms($key, $value);
    if ($match != TRUE) {
      create_term($key, $value);
    } 
    $result = db_query("select t.* from {term_data} t where t.vid = %d and t.name = '%s'", $value, $key);
    while ($row = db_fetch_array($result)) {
      $tid = ($row['tid']);
      $newnode->taxonomy[$value][$tid] = $tid;
    }
  }
}
  
/**
 * camtasia_relay_node_create function
 * returns node object.
 * @param string $xmldata
 * @param string $fileName
 * @param string $uid
 * @param string $status
 * @param string $promote
 * @param string $type
 * @return $newnode
 */
function camtasia_relay_node_create($xmldata, $xml_file_path, $uid, $status, $promote, $type) {
  $newnode = new stdClass();
  $newnode->title = $xmldata->title;
  if (!empty($xmldata->description)) {
    $newnode->body = $xmldata->description;
  }
  else {
    $newnode->body = 'Description Not Provided';
  }
  $newnode->date = $xmldata->date;
  $newnode->profile = $xmldata->profile;
  $newnode->display_name = $xmldata->presenter->displayName;
  $newnode->user_name = $xmldata->presenter->userName;
  $newnode->email = $xmldata->presenter->email;
  
  $newnode->recorder_display_name = $xmldata->recordedBy->displayName;
  $newnode->recorder_email = $xmldata->recordedBy->email;
  
  $newnode->trimmed_duration = $xmldata->outputFiles->duration;
  $camtasia_relay_default_user = variable_get('camtasia_relay_default_user', '');
  $mail_Opt = variable_get('camtasia_relay_node_mail', '');

  if (empty($camtasia_relay_default_user)) {
    $user_name = $newnode->user_name;
  }
  else {
    $user_name = $camtasia_relay_default_user;
  }
  $uid = camtasia_relay_user_check($user_name, $newnode->email, $mail_Opt);
  global $user;
  $user = user_load($uid);
  $newnode->uid = $uid;
  $filesize = filesize($xml_file_path);
  $des_file = camtasia_relay_file_move($xml_file_path, $newnode->user_name, $newnode->profile);

  camtasia_relay_file_attach($des_file, $filesize, $newnode);
  $xml_files = $xmldata->outputFiles->fileList->file;
  foreach ($xml_files as $xml_file) {
    $output_file_name = $xml_file['name'];
    $output_file_path = dirname($xml_file_path) ."/". $output_file_name;
    if (file_exists($output_file_path)) {
      $filesize = filesize($output_file_path);
      $des_file = camtasia_relay_file_move($output_file_path, $newnode->user_name, $newnode->profile);
      camtasia_relay_file_attach($des_file, $filesize, $newnode);
    }
  }
  //  decide whether to publish or un publish the node
  //  http://drupal.org/node/49768
  //  $node->status   unpublished/published (0|1)
  $camtasia_relay_publish = variable_get('camtasia_relay_node_publish', '');
  if ($camtasia_relay_publish==0) {
    //  publish.
    $newnode->status = 1;
  }
  else {
    //  un publish
    $newnode->status = 0;
  }

  $newnode->type = $type;
  $newnode->promote = $promote;
  return $newnode;
}

function camtasia_relay_file_attach( $file_path_des, $filesize, $node_obj) {
  $name = basename($file_path_des);
  $mime = file_get_mimetype($name);
  $file = new stdClass();
  $file->filename = $name;
  $file->filepath = $file_path_des;
  $file->filemime = $mime;
  $file->filesize = $filesize;
  $file_obj->filesource = $name;

  $file->uid = $node_obj->uid;
  $file->status = FILE_STATUS_PERMANENT;
  $file->timestamp = time();
  $file->list = 1;
  $file->new = TRUE;

  drupal_write_record('files', $file);
  // Attach the file object to your node
  $err_msg = $file->fid;
  watchdog('camtasia_relay', 'file'. $err_msg , NULL, WATCHDOG_ERROR, NULL);
  $node_obj->files[$file->fid] = $file;
}

function camtasia_relay_file_move($src_file_path, $user_name, $profile) {
  $user_name = str_ireplace(' ', '_', $user_name);
  $profile = str_ireplace(' ', '_', $profile);
  $info = pathinfo($src_file_path);

  //$files_dir = $_SERVER['DOCUMENT_ROOT'] . base_path() . file_directory_path() ."/";
  $files_dir = realpath(".") . "/". file_directory_path() ."/";

  $camtasia_relay_dir = $files_dir . 'camtasia_relay1/';
  $user_dir = $camtasia_relay_dir . $user_name .'/';
  $profile_dir = $user_dir . $profile .'/';

  $ext_dir = $profile_dir;

  $des_file_path = $ext_dir . '/'. basename($src_file_path);
  camtasia_relay_mkdir_r($ext_dir);
  file_move($src_file_path, $des_file_path,  FILE_EXISTS_REPLACE);
  $relative_file_path = 'camtasia_relay1/' . $user_name .'/'. $profile . '/' . basename($src_file_path);
  return $relative_file_path;
}

function camtasia_relay_user_check($user_name, $user_mail, $send_mail) {
  //  Check for the existence of current user $newnode->user_name
  //  Create if not exists
  //  put uid accordingly $newnode->uid
  $result = db_query("SELECT * FROM {users} WHERE name = '%s'", $user_name);

  $row = db_fetch_array($result);
  if (!empty($row)) {
    //  user exists
    return $row['uid'];
  }
  else {
    //  user not exists
    //  So create user
    //  send mail to user email id
    //  Drupal User module function that generates MD5 hash password
    $pass = user_password();
    $role_id = variable_get('camtasia_relay_user_roles', DRUPAL_AUTHENTICATED_RID);
    $role_name = db_result(db_query('SELECT name FROM {role} WHERE rid = %d', $role_id));
    //'access' => 0, /* optional, i didnt want user to be able to use this account yet*/
    $roles = array($role_id => $role_name);
    $details = array(
      'name' => $user_name,
      'pass' => $pass,
      'mail' => $user_mail,
      'status' => 1,
      'roles' => $roles,
      );
    
    $user = user_save(null, $details);
    if ($send_mail==0) {
      //	yes
      _user_mail_notify('register_admin_created', $user,  NULL);
    }
  }   //  else
  return $user->uid;  
}